--_________________________________________________________________________________________________
--                                                                                                |
--                                           |TDC core|                                           |
--                                                                                                |
--                                         CERN,BE/CO-HT                                          |
--________________________________________________________________________________________________|

---------------------------------------------------------------------------------------------------
--                                                                                                |
--                                          data_engine                                           |
--                                                                                                |
---------------------------------------------------------------------------------------------------
-- File         data_engine.vhd                                                                   |
--                                                                                                |
-- Description  Bit messy way that loops trough the following sequence:
--                - waits for the activation from PCIe/ VME interface                             |
--                - sends out an ID_DAT frame asking for an identification variable (the agent's  |
--                  address is defined trough a dedicated PCIe/ VME register)                     |
--                - waits for the reply and stores the received data to registers accessible      |
--                  through PCIe/ VME                                                             |
--                - sends out an ID_DAT frame for a consumed variable (the agent's address is     |
--                  defined trough a dedicated PCIe/ VME register)                                |
--                - sends out the corresponding RP_DAT frame to be consumed by the agent          |
--                - sends out a number of bourrage frames that keep the WorldFIP line active (the |
--                  number of those frames is defined trough a dedicated PCIe/ VME register)      |
--                                                                                                |
-- Authors      Evangelia Gousiou (Evangelia.Gousiou@cern.ch)                                     |
-- Date         07/2014                                                                           |
-- Version      v1                                                                                |
-- Depends on                                                                                     |
--                                                                                                |
----------------                                                                                  |
-- Last changes                                                                                   |
--     07/2014  v1  EG  First version                                                             |
--                                                                                                |
---------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------
--                               GNU LESSER GENERAL PUBLIC LICENSE                                |
--                              ------------------------------------                              |
-- This source file is free software; you can redistribute it and/or modify it under the terms of |
-- the GNU Lesser General Public License as published by the Free Software Foundation; either     |
-- version 2.1 of the License, or (at your option) any later version.                             |
-- This source is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;       |
-- without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      |
-- See the GNU Lesser General Public License for more details.                                    |
-- You should have received a copy of the GNU Lesser General Public License along with this       |
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html                     |
---------------------------------------------------------------------------------------------------



--=================================================================================================
--                                       Libraries & Packages
--=================================================================================================

-- Standard library
library IEEE;
use IEEE.std_logic_1164.all; -- std_logic definitions
use IEEE.NUMERIC_STD.all;    -- conversion functions
-- Specific library
library work;
use work.masterFIP_pkg.all;   -- definitions of types, constants, entities


--=================================================================================================
--                            Entity declaration for data_engine
--=================================================================================================

entity data_engine is
  port
  -- INPUTS
     -- Signals from the clk_rst_manager
    (clk_i                : in std_logic;                     -- 40 MHz
     rst_i                : in std_logic;                     -- global reset

     -- Signals from the reg_ctrl unit: communication with GN4124/VME for registers configuration
     activate_loop_p_i    : in std_logic;                     -- activates communication loop
     continuous_loop_i    : in std_logic;                     -- 
     num_of_bourr_i       : in std_logic_vector(31 downto 0);

    --  --  --  --  --  --  --  --  --  --  --  --  --  --  -- --  --  --  --  --  --  --  --  --
    -- Signal from the wf_fd_transmitter unit

    tx_completed_p_i           : in std_logic;                     -- pulse upon termination of a
                                                                   -- produced RP_DAT transmission

    tx_byte_request_p_i        : in std_logic;                     -- used for the counting of the
                                                                   -- # produced bytes

    --  --  --  --  --  --  --  --  --  --  --  --  --  --  -- --  --  --  --  --  --  --  --  --
    -- Signals from the wf_fd_receiver unit

    rx_byte_ready_p_i          : in std_logic;

    rx_fss_crc_fes_ok_p_i      : in std_logic; -- indication of a frame (ID_DAT or RP_DAT) with
                                               -- correct FSS, FES and CRC

    rx_crc_wrong_p_i           : in std_logic; -- indication of a frame with a wrong CRC
                                               -- pulse upon FES detection

    rx_fss_received_p_i        : in std_logic; -- pulse upon FSS detection (ID/ RP_DAT)
															  
  -- OUTPUTS

    irq_p_o                    : out std_logic;-- irq upon the end of a loop
    --  --  --  --  --  --  --  --  --  --  --  --  --  --  -- --  --  --  --  --  --  --  --  --
    -- Signal to the wf_fd_transmitter unit
    tx_start_p_o               : out std_logic; -- launches the transmitter
    tx_byte_request_accept_p_o : out std_logic; -- answer to tx_byte_request_p_i
    tx_last_data_byte_p_o      : out std_logic; -- indication of the last data-byte
                                                -- (CRC & FES not included)

    --  --  --  --  --  --  --  --  --  --  --  --  --  --  -- --  --  --  --  --  --  --  --  --
    -- Signal to the wf_fd_receiver
    rx_rst_o                   : out std_logic; -- reset during production or
                                                -- reset pulse when during reception a frame is rejected
                                                -- (example: ID_DAT > 8 bytes, RP_DAT > 133 bytes, 
                                                --  wrong ID_DAT CTRL, variable, subs bytes)

    --  --  --  --  --  --  --  --  --  --  --  --  --  --  -- --  --  --  --  --  --  --  --  --
    -- Signal to the wf_consumption unit
    cons_bytes_excess_o        : out std_logic; -- indication of a consumed RP_DAT frame with more
                                                -- than 133 bytes

    --  --  --  --  --  --  --  --  --  --  --  --  --  --  -- --  --  --  --  --  --  --  --  --
    -- Signals to the wf_production & wf_consumption
    prod_byte_index_o          : out std_logic_vector (7 downto 0); -- index of the byte being
                                                                    -- produced
    cons_byte_index_o          : out std_logic_vector (7 downto 0); -- index of the byte being
                                                                    -- consumed

    --  --  --  --  --  --  --  --  --  --  --  --  --  --  -- --  --  --  --  --  --  --  --  --
    -- Signals to the wf_production, wf_consumption, wf_reset_unit
    var_o                      : out t_var);   -- received variable; takes a value only after a
                                               -- valid ID_DAT frame with SUBS the station's address


end data_engine;


--=================================================================================================
--                                    architecture declaration
--=================================================================================================

architecture rtl of data_engine is

  type engine_state_ty is (IDLE, START_SENDING, SEND_ID_DAT_IDENT, WAIT_FSS, RST_RX, SEND_ID_DAT_BOURR, BOURR_COUNT,
                           WAIT_TURNAR_TIME_BOURR, GENER_IRQ, SEND_RP_DAT_CONSU, WAIT_TURNAR_TIME_ID_RP,
                           SEND_ID_DAT_CONSU, WAIT_TURNAR_TIME_RP_ID, RECEIVE_RP_DAT_IDENT, START_SENDING2);
  signal engine_st, nxt_engine_st    : engine_state_ty;

  signal config_adr_c                : unsigned(7 downto 0);

  signal time_c_full_p, time_c_en    : std_logic;
  signal time_c_rst                  : std_logic;
  signal time_c                      : std_logic_vector(31 downto 0);


  signal s_id_dat_ctrl_byte, s_id_dat_var_byte, s_id_dat_frame_ok        : std_logic;
  signal s_cons_wait_FSS                                      : std_logic;
  signal s_prod_wait_turnar_time                                          : std_logic;
  -- variable identification
  signal s_var_aux, s_var                                                              : t_var;
  signal s_var_identified, s_broadcast_var                                             : std_logic;
  signal s_prod_or_cons                                            : std_logic_vector (1 downto 0);
  -- time counters (turnaround, silence, timeout)
  signal s_time_c_top, s_turnaround_time, s_silence_time, s_turnaround_time_bourr                  : unsigned (17 downto 0);
  signal s_time_c_load, s_time_c_is_zero                                               : std_logic;
  signal s_session_timedout                                                            : std_logic;
  -- received & produced byte counters
  signal s_rx_bytes_c, s_prod_bytes_c                                      : unsigned (7 downto 0);
  signal s_prod_bytes_c_rst, s_prod_bytes_c_inc                                        : std_logic;
  signal s_rx_bytes_c_rst, s_rx_bytes_c_inc                                            : std_logic;
  -- transmitter controls
  signal s_tx_start_prod_p, s_tx_byte_request_accept_p, s_tx_byte_request_accept_p_d1  : std_logic;
  signal s_tx_byte_request_accept_p_d2, s_tx_last_data_byte_p, s_tx_last_data_byte_p_d : std_logic;
  -- length of produced data
  signal s_prod_data_lgth                                          : std_logic_vector (7 downto 0);
  signal s_prod_data_lgth_match                                                        : std_logic;

  signal s_idle_state, s_wait_turnar_time, s_wait_FSS, s_consuming, s_rst_rx_p, s_send_id_dat_ident         : std_logic;
  signal s_producing, irq_p                                             : std_logic;
  signal s_bourr_c                                                 : unsigned (7 downto 0);
  signal s_bourr_c_rst, s_bourr_c_inc, s_tx_id_dat_p                                                 : std_logic;
  signal s_time_c :unsigned(17 downto 0);


  -- Chipscope
  component chipscope_ila
    port (
      CONTROL : inout std_logic_vector(35 downto 0);
      CLK     : in    std_logic;
      TRIG0   : in    std_logic_vector(31 downto 0);
      TRIG1   : in    std_logic_vector(31 downto 0);
      TRIG2   : in    std_logic_vector(31 downto 0);
      TRIG3   : in    std_logic_vector(31 downto 0));
  end component;
  component chipscope_icon
    port (
      CONTROL0 : inout std_logic_vector (35 downto 0));
  end component;
  signal CONTROL : std_logic_vector(35 downto 0);
  signal CLK     : std_logic;
  signal TRIG0   : std_logic_vector(31 downto 0);
  signal TRIG1   : std_logic_vector(31 downto 0);
  signal TRIG2   : std_logic_vector(31 downto 0);
  signal TRIG3   : std_logic_vector(31 downto 0);


--=================================================================================================
--                                       architecture begin
--=================================================================================================
begin


---------------------------------------------------------------------------------------------------
--                                             FSM                                               --
---------------------------------------------------------------------------------------------------

--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
-- data_engine_fsm_seq FSM: the state machine is divided in three parts (a clocked process
-- to store the current state, a combinatorial process to manage state transitions and finally a
-- combinatorial process to manage the output signals), which are the three processes that follow.

--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
-- Synchronous process: storage of the current state of the FSM

  data_engine_fsm_seq: process (clk_i)
  begin
    if rising_edge (clk_i) then
      if rst_i ='1' then
        engine_st <= IDLE;
      else
        engine_st <= nxt_engine_st;
      end if;
    end if;
  end process;
    
--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
-- Combinatorial process
  data_engine_fsm_comb: process (engine_st, activate_loop_p_i, continuous_loop_i, tx_completed_p_i,
                                 rx_fss_received_p_i, time_c, time_c_full_p, s_bourr_c, s_rx_bytes_c,
                                 s_time_c_is_zero, rx_fss_crc_fes_ok_p_i, rx_crc_wrong_p_i, s_time_c, num_of_bourr_i)
  begin
    case engine_st is

      --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
      -- from the IDLE state modifications/readings of the configuration can be initiated;
      when IDLE =>
                  s_idle_state            <= '1';
                  s_wait_turnar_time      <= '0';
                  s_wait_FSS              <= '0';
                  s_consuming             <= '0';
                  s_rst_rx_p              <= '0';
                  s_producing             <= '0';
                  irq_p                   <= '0';
                  s_var                   <= var_whatever;
                  s_bourr_c_inc           <= '0';
                  s_tx_id_dat_p           <= '0';
            
                  if activate_loop_p_i = '1' or  continuous_loop_i = '1' then   -- loop activation or continuous loop
                    nxt_engine_st         <= START_SENDING;
                  else
                    nxt_engine_st         <= IDLE;
                  end if;

      when START_SENDING =>
                  s_idle_state            <= '0';
                  s_wait_turnar_time      <= '0';
                  s_wait_FSS              <= '0';
                  s_consuming             <= '0';
                  s_rst_rx_p              <= '0';
                  s_producing             <= '0';
                  irq_p                   <= '0';
                  s_var                   <= var_whatever;
                  s_bourr_c_inc           <= '0';
                  s_tx_id_dat_p           <= '1';
            
                  nxt_engine_st         <= SEND_ID_DAT_IDENT;


      when SEND_ID_DAT_IDENT => 
                  s_idle_state            <= '0';
                  s_wait_turnar_time      <= '0';
                  s_wait_FSS              <= '0';
                  s_consuming             <= '0';
                  s_rst_rx_p              <= '0';
                  s_producing             <= '1';
                  irq_p                   <= '0';
                  s_var                   <= id_dat_ident;
                  s_bourr_c_inc           <= '0';
                  s_tx_id_dat_p           <= '0';
            
                  if tx_completed_p_i = '1' then
                    nxt_engine_st         <= WAIT_FSS;
                  else
                    nxt_engine_st         <= SEND_ID_DAT_IDENT;
                  end if;

      when WAIT_FSS => -- wait FSS to arrive 
                  s_idle_state            <= '0';
                  s_wait_turnar_time      <= '0';
                  s_wait_FSS              <= '1';
                  s_consuming             <= '0';
                  s_rst_rx_p              <= '0';
                  s_producing             <= '0';
                  irq_p                   <= '0';
                  s_var                   <= var_whatever;
                  s_bourr_c_inc           <= '0';
                  s_tx_id_dat_p           <= '0';
            
                  if rx_fss_received_p_i = '1' then
                    nxt_engine_st         <= RECEIVE_RP_DAT_IDENT;
                  elsif s_time_c_is_zero = '1' then
                    nxt_engine_st         <= START_SENDING2; 
                  else
                    nxt_engine_st         <= WAIT_FSS;
                  end if;

      when RECEIVE_RP_DAT_IDENT => 
                  s_idle_state            <= '0';
                  s_wait_turnar_time      <= '0';
                  s_wait_FSS              <= '0';
                  s_consuming             <= '1';
                  s_rst_rx_p              <= '0';
                  s_producing             <= '0';
                  irq_p                   <= '0';
                  s_var                   <= rp_dat_ident;
                  s_bourr_c_inc           <= '0';
                  s_tx_id_dat_p           <= '0';
            
                  if (rx_fss_crc_fes_ok_p_i = '1') or (rx_crc_wrong_p_i = '1') then 
                    nxt_engine_st         <= WAIT_TURNAR_TIME_RP_ID;
                  elsif (s_rx_bytes_c > c_MAX_FRAME_BYTES) then 
                    nxt_engine_st         <= RST_RX; 
                  else
                    nxt_engine_st         <= RECEIVE_RP_DAT_IDENT;
                  end if;


      when WAIT_TURNAR_TIME_RP_ID =>
                  s_idle_state            <= '0';
                  s_wait_turnar_time      <= '1';
                  s_wait_FSS              <= '0';
                  s_consuming             <= '0';
                  s_rst_rx_p              <= '0';
                  s_producing             <= '0';
                  irq_p                   <= '0';
                  s_var                   <= var_whatever;
                  s_bourr_c_inc           <= '0';
                  s_tx_id_dat_p           <= '0';
            
                  if s_time_c_is_zero = '1' then
                    nxt_engine_st         <= SEND_ID_DAT_CONSU;
                  else
                    nxt_engine_st         <= WAIT_TURNAR_TIME_RP_ID;
                  end if;

      when START_SENDING2 =>
                  s_idle_state            <= '0';
                  s_wait_turnar_time      <= '0';
                  s_wait_FSS              <= '0';
                  s_consuming             <= '0';
                  s_rst_rx_p              <= '0';
                  s_producing             <= '0';
                  irq_p                   <= '0';
                  s_var                   <= var_whatever;
                  s_bourr_c_inc           <= '0';
                  s_tx_id_dat_p           <= '1';
            
                  nxt_engine_st         <= SEND_ID_DAT_CONSU;


      when SEND_ID_DAT_CONSU =>
                  s_idle_state            <= '0';
                  s_wait_turnar_time      <= '0';
                  s_wait_FSS              <= '0';
                  s_consuming             <= '0';
                  s_rst_rx_p              <= '0';
                  s_producing             <= '1';
                  irq_p                   <= '0';
                  s_var                   <= id_dat_consu;
                  s_bourr_c_inc           <= '0';
                  s_tx_id_dat_p           <= '0';
            
                  if tx_completed_p_i = '1' then
                    nxt_engine_st         <= WAIT_TURNAR_TIME_ID_RP;
                  else
                    nxt_engine_st         <= SEND_ID_DAT_CONSU;
                  end if;

      when WAIT_TURNAR_TIME_ID_RP =>
                  s_idle_state            <= '0';
                  s_wait_turnar_time      <= '1';
                  s_wait_FSS              <= '0';
                  s_consuming             <= '0';
                  s_rst_rx_p              <= '0';
                  s_producing             <= '0';
                  irq_p                   <= '0';
                  s_var                   <= var_whatever;
                  s_bourr_c_inc           <= '0';
                  s_tx_id_dat_p           <= '0';
            
                  if s_time_c_is_zero = '1' then
                    nxt_engine_st         <= SEND_RP_DAT_CONSU;
                  else
                    nxt_engine_st         <= WAIT_TURNAR_TIME_ID_RP;
                  end if;

      when SEND_RP_DAT_CONSU =>
                  s_idle_state            <= '0';
                  s_wait_turnar_time      <= '0';
                  s_wait_FSS              <= '0';
                  s_consuming             <= '0';
                  s_rst_rx_p              <= '0';
                  s_producing             <= '1';
                  irq_p                   <= '0';
                  s_var                   <= rp_dat_consu;
                  s_bourr_c_inc           <= '0';
                  s_tx_id_dat_p           <= '0';
            
                  if tx_completed_p_i = '1' then
                    nxt_engine_st         <= GENER_IRQ;
                  else
                    nxt_engine_st         <= SEND_RP_DAT_CONSU;
                  end if;

      when GENER_IRQ =>
                  s_idle_state            <= '0';
                  s_wait_turnar_time      <= '0';
                  s_wait_FSS              <= '0';
                  s_consuming             <= '0';
                  s_rst_rx_p              <= '0';
                  s_producing             <= '0';
                  s_var                   <= var_whatever;
                  irq_p                   <= '1';
                  nxt_engine_st           <= WAIT_TURNAR_TIME_BOURR;
                  s_bourr_c_inc           <= '0';
                  s_tx_id_dat_p           <= '0';

      when WAIT_TURNAR_TIME_BOURR =>
                  s_idle_state            <= '0';
                  s_wait_turnar_time      <= '1';
                  s_wait_FSS              <= '0';
                  s_consuming             <= '0';
                  s_rst_rx_p              <= '0';
                  s_producing             <= '0';
                  irq_p                   <= '0';
                  s_var                   <= var_whatever;
                  s_tx_id_dat_p           <= '0';
            
                  if s_time_c = 1 then
                    s_bourr_c_inc         <= '1';
                    if s_bourr_c > unsigned(num_of_bourr_i(7 downto 0)) then
                      nxt_engine_st       <= IDLE;
                    else
                      nxt_engine_st       <= WAIT_TURNAR_TIME_BOURR;
				    end if;
                  else
                    s_bourr_c_inc         <= '0';
                    if s_time_c_is_zero = '1' then
                      nxt_engine_st         <= SEND_ID_DAT_BOURR;
                    else
                      nxt_engine_st         <= WAIT_TURNAR_TIME_BOURR;
                    end if;
                  end if;

      when BOURR_COUNT =>
                  s_idle_state            <= '0';
                  s_wait_turnar_time      <= '0';
                  s_wait_FSS              <= '0';
                  s_consuming             <= '0';
                  s_rst_rx_p              <= '0';
                  s_producing             <= '0';
                  irq_p                   <= '0';
                  s_bourr_c_inc           <= '1';
                  s_var                   <= var_whatever;

                  if s_bourr_c > unsigned(num_of_bourr_i(7 downto 0)) then
                    s_tx_id_dat_p         <= '0';
                    nxt_engine_st         <= IDLE;
                  else
                    s_tx_id_dat_p         <= '1';
                    nxt_engine_st         <= SEND_ID_DAT_BOURR;
                  end if;

      when SEND_ID_DAT_BOURR =>
                  s_idle_state            <= '0';
                  s_wait_turnar_time      <= '0';
                  s_wait_FSS              <= '0';
                  s_consuming             <= '0';
                  s_rst_rx_p              <= '0';
                  s_producing             <= '1';
                  s_var                   <= id_dat_bourr;
                  irq_p                   <= '0';
                  s_bourr_c_inc           <= '0';
                  s_tx_id_dat_p           <= '0';

                  if tx_completed_p_i = '1' then
                    nxt_engine_st           <= WAIT_TURNAR_TIME_BOURR;
                  else
                    nxt_engine_st           <= SEND_ID_DAT_BOURR;
                  end if;

      when RST_RX =>
                  s_idle_state            <= '0';
                  s_wait_turnar_time      <= '0';
                  s_wait_FSS              <= '0';
                  s_consuming             <= '0';
                  s_rst_rx_p              <= '1';
                  s_producing             <= '0';
                  s_var                   <= var_whatever;
                  irq_p                   <= '0';
                  s_bourr_c_inc           <= '0';
                  s_tx_id_dat_p           <= '0';

                  nxt_engine_st           <= IDLE;

      --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
      when others =>
                  s_idle_state            <= '0';
                  s_wait_turnar_time      <= '0';
                  s_wait_FSS              <= '0';
                  s_consuming             <= '0';
                  s_rst_rx_p              <= '0';
                  s_producing             <= '0';
                  s_send_id_dat_ident     <= '0';
                  irq_p                   <= '0';
                  s_var                   <= var_whatever;
                  s_bourr_c_inc           <= '0';
                  s_tx_id_dat_p           <= '0';

                  nxt_engine_st           <= IDLE;
        end case;
    end process;

  var_o   <= s_var;
  irq_p_o <= irq_p;

--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
-- Instantiation of the wf_prod_data_lgth_calc unit that calculates the amount of bytes that have
-- to be transmitted when a variable is produced; the CTRL, MPS and nanoFIP_status bytes are
-- included; The FSS, CRC and FES bytes are not included!

  Produced_Data_Length_Calculator: wf_prod_data_lgth_calc
  port map(
    uclk_i             => clk_i,
    nfip_rst_i         => rst_i,
    p3_lgth_i          => "000",
    var_i              => s_var,
    -------------------------------------------------------
    prod_data_lgth_o   => s_prod_data_lgth);
    -------------------------------------------------------


--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
-- Instantiation of a wf_incr_counter for the counting of the number of the bytes that are
-- being produced. The same counter is used for the ID_DAT and RP_DAT that are being produced.
-- The counter is reset at the "PRODUCE_WAIT_TURNAR_TIME" state of the FSM and
-- counts bytes following the "tx_byte_request_p_i" pulse in the "PRODUCE" state.

  Prod_Bytes_Counter: wf_incr_counter
  generic map(g_counter_lgth => 8)
  port map(
    uclk_i            => clk_i,
    counter_reinit_i  => s_prod_bytes_c_rst,
    counter_incr_i    => s_prod_bytes_c_inc,
    counter_is_full_o => open,
    -------------------------------------------------------
    counter_o         => s_prod_bytes_c);
    -------------------------------------------------------

  --  --  --  --  --  --  --  --  --  --  --
  s_prod_bytes_c_rst <= '0'                 when s_producing = '1' else '1';
  s_prod_bytes_c_inc <= tx_byte_request_p_i when s_producing = '1' else '0';

  -- when s_prod_data_lgth bytes have been counted,the signal s_prod_data_lgth_match is activated
  s_prod_data_lgth_match <= '1' when s_prod_bytes_c = unsigned (s_prod_data_lgth) else '0';


--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
-- Instantiation of a wf_incr_counter for the counting of the number of bytes that are being
-- received. The counter is reset at the "WAIT_FSS" state and counts bytes following the
-- "rx_byte_ready_p_i" pulse in the "CONSUME" state.

  Rx_Bytes_Counter: wf_incr_counter
  generic map(g_counter_lgth => 8)
  port map(
    uclk_i            => clk_i,
    counter_reinit_i  => s_rx_bytes_c_rst,
    counter_incr_i    => s_rx_bytes_c_inc,
    counter_is_full_o => open,
    -------------------------------------------------------
    counter_o         => s_rx_bytes_c);
    -------------------------------------------------------

  --  --  --  --  --  --  --  --  --  --  --
  s_rx_bytes_c_rst   <= '0'               when (s_consuming = '1') else '1';
  s_rx_bytes_c_inc   <= rx_byte_ready_p_i when (s_consuming = '1') else '0';


  Num_of_Bourrage_Counter: wf_incr_counter
  generic map(g_counter_lgth => 8)
  port map(
    uclk_i            => clk_i,
    counter_reinit_i  => s_idle_state,
    counter_incr_i    => s_bourr_c_inc,
    counter_is_full_o => open,
    -------------------------------------------------------
    counter_o         => s_bourr_c);
    -------------------------------------------------------



---------------------------------------------------------------------------------------------------
--                                  Independent Timeout Counter                                  --
---------------------------------------------------------------------------------------------------

-- Instantiation of a wf_decr_counter relying only on the system clock as an additional
-- way to go back to IDLE state, in case any other logic is being stuck.

  Session_Timeout_Counter: wf_decr_counter
  generic map(g_counter_lgth => c_SESSION_TIMEOUT_C_LGTH)
  port map(
    uclk_i            => clk_i,
    counter_rst_i     => rst_i,
    counter_top_i     => (others => '1'),
    counter_load_i    => s_idle_state,
    counter_decr_i    => '1', -- on each uclk tick
    counter_o         => open,
    ---------------------------------------------------
    counter_is_zero_o => s_session_timedout);
    ---------------------------------------------------



---------------------------------------------------------------------------------------------------
--                                   Turnaround & Silence times                                  --
---------------------------------------------------------------------------------------------------

--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
-- retrieval of the turnaround and silence times (in equivalent number of uclk ticks) from the
-- c_TIMEOUTS_TABLE declared in the masterFIP_pkg unit.

  s_turnaround_time       <= ("000000001000110000");--("000001000110000000");

  s_turnaround_time_bourr <= ("000001101101011000");--("000000000110110101");

  s_silence_time          <= ("000001011101110000");--("000000101110111000");

--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
-- Instantiation of a wf_decr_counter for the counting of turnaround and silence times.
-- The same counter is used in both cases. The signal s_time_c_top initializes the counter
-- to either the turnaround or the silence time. If after the correct arrival of an ID_DAT frame
-- the identified variable is a produced one the counter loads to the turnaround time, whereas if
-- it had been a consumed variable it loads to the silence. The counting takes place during the
-- states "PRODUCE_WAIT_TURNAR_TIME" and "CONSUME_WAIT_FSS" respectively.

  Turnaround_and_Silence_Time_Counter: wf_decr_counter
  generic map(g_counter_lgth => 18)
  port map(
    uclk_i            => clk_i,
    counter_rst_i     => rst_i,
    counter_top_i     => s_time_c_top,
    counter_load_i    => s_time_c_load,
    counter_decr_i    => '1', -- on each uclk tick
    counter_o         => s_time_c,
    -------------------------------------------------------
    counter_is_zero_o => s_time_c_is_zero);
    -------------------------------------------------------

  --  --  --  --  --  --  --  --  --  --  --
  s_time_c_top  <= s_turnaround_time when (s_consuming = '1' or s_var = id_dat_consu) else s_turnaround_time_bourr when irq_p = '1' else s_silence_time;
  s_time_c_load <= '0'               when s_wait_turnar_time = '1' or s_wait_FSS = '1' else '1';


---------------------------------------------------------------------------------------------------
--                                     Signals Registration                                      --
---------------------------------------------------------------------------------------------------

  process (clk_i)
  begin
    if rising_edge (clk_i) then
      if rst_i = '1' then
        tx_last_data_byte_p_o         <= '0';
        s_tx_last_data_byte_p_d       <= '0';
        s_tx_byte_request_accept_p_d1 <= '0';
        s_tx_byte_request_accept_p_d2 <= '0';
        s_tx_start_prod_p             <= '0';

      else
        s_tx_last_data_byte_p_d       <= s_tx_last_data_byte_p;
        tx_last_data_byte_p_o         <= s_tx_last_data_byte_p_d;
        s_tx_byte_request_accept_p_d1 <= s_tx_byte_request_accept_p;
        s_tx_byte_request_accept_p_d2 <= s_tx_byte_request_accept_p_d1;
        s_tx_start_prod_p             <= (s_wait_turnar_time and s_time_c_is_zero) or s_tx_id_dat_p;
      end if;
    end if;
  end process;

  --  --  --  --  --  --  --  --  --  --  --
  s_tx_byte_request_accept_p <= s_producing and (tx_byte_request_p_i or s_tx_start_prod_p);
  s_tx_last_data_byte_p      <= s_producing and s_prod_data_lgth_match and tx_byte_request_p_i;

-- response to wf_tx_serializer request for a byte
  tx_byte_request_accept_p_o <= s_tx_byte_request_accept_p_d2;

-- production starts after the expiration of the turnaround time
  tx_start_p_o               <= s_tx_start_prod_p;

-- index of the byte being produced/ consumed
  prod_byte_index_o          <= std_logic_vector (s_prod_bytes_c);
  cons_byte_index_o          <= std_logic_vector (s_rx_bytes_c);

  rx_rst_o                   <= '1' when (s_rst_rx_p = '1')              or
                                         (s_producing = '1') else '0';


---------------------------------------------------------------------------------------------------
--                                           CHIPSCOPE                                           --
---------------------------------------------------------------------------------------------------   
  
   chipscope_ila_1 : chipscope_ila
   port map (
     CONTROL => CONTROL,
     CLK     => clk_i,
     TRIG0   => TRIG0,
     TRIG1   => TRIG1,
     TRIG2   => TRIG2,
     TRIG3   => TRIG3);

  chipscope_icon_1 : chipscope_icon
   port map (
     CONTROL0 => CONTROL);
 
  TRIG0(0)            <= activate_loop_p_i;
  TRIG0(1)            <= continuous_loop_i;
  TRIG0(2)            <= s_tx_start_prod_p;
  TRIG0(3)            <= tx_completed_p_i;
  TRIG0(4)            <= rx_fss_received_p_i;

  TRIG0(5)            <= s_time_c_is_zero;
  TRIG0(6)            <= s_producing;
  TRIG0(7)            <= rx_fss_crc_fes_ok_p_i;
  TRIG0(8)            <= irq_p;

  TRIG0(9)            <= '1' when s_time_c = 1 else '0';
  TRIG0(10)           <= s_bourr_c_inc;
  TRIG0(18 downto 11) <= std_logic_vector(s_bourr_c);
  TRIG0(26 downto 19) <= num_of_bourr_i(7 downto 0);
  TRIG0(27)           <= s_wait_turnar_time;

  TRIG1(17 downto 0)  <= std_logic_vector(s_time_c);

  TRIG1(25 downto 18) <= std_logic_vector(s_prod_bytes_c);

end architecture rtl;
--=================================================================================================
--                                        architecture end
--=================================================================================================
---------------------------------------------------------------------------------------------------
--                                      E N D   O F   F I L E
---------------------------------------------------------------------------------------------------
