--_________________________________________________________________________________________________
--                                                                                                |
--                                           |TDC core|                                           |
--                                                                                                |
--                                         CERN,BE/CO-HT                                          |
--________________________________________________________________________________________________|

---------------------------------------------------------------------------------------------------
--                                                                                                |
--                                      fmc_masterFIP_core                                        |
--                                                                                                |
---------------------------------------------------------------------------------------------------
-- File         fmc_masterFIP_core.vhd                                                            |
--                                                                                                |
-- Description  The masterFIP core top level instantiates all the modules needed to provide
--                                                                                                |
-- Authors      Evangelia Gousiou (Evangelia.Gousiou@cern.ch)                                     |
-- Date         07/2014                                                                           |
-- Version      v1                                                                                |
-- Depends on                                                                                     |
--                                                                                                |
----------------                                                                                  |
-- Last changes                                                                                   |
--     07/2014  v1  EG  First version                                                             |
--                                                                                                |
---------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------
--                               GNU LESSER GENERAL PUBLIC LICENSE                                |
--                              ------------------------------------                              |
-- This source file is free software; you can redistribute it and/or modify it under the terms of |
-- the GNU Lesser General Public License as published by the Free Software Foundation; either     |
-- version 2.1 of the License, or (at your option) any later version.                             |
-- This source is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;       |
-- without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      |
-- See the GNU Lesser General Public License for more details.                                    |
-- You should have received a copy of the GNU Lesser General Public License along with this       |
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html                     |
---------------------------------------------------------------------------------------------------

--=================================================================================================
--                                       Libraries & Packages
--=================================================================================================
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use work.masterFIP_pkg.all;
use work.gencores_pkg.all;
use work.wishbone_pkg.all;
--use work.wrcore_pkg.all;
use work.genram_pkg.all;


--=================================================================================================
--                                Entity declaration for fmc_masterFIP_core
--=================================================================================================
entity fmc_masterFIP_core is
  generic
    (g_span           : integer := 32;                    -- address span in bus interfaces
     g_width          : integer := 32;                    -- data width in bus interfaces
     values_for_simul : boolean := FALSE);                -- this generic is set to TRUE
                                                          -- when instantiated in a test-bench
  port
    (-- Clock and reset
     clk_i       : in    std_logic;                       -- 
     rst_i       : in    std_logic;                       -- 

      -- FielDrive
     fd_rxcdn_i  : in    std_logic;
     fd_rxd_i    : in    std_logic;
     fd_txer_i   : in    std_logic;
     fd_wdgn_i   : in    std_logic;
     fd_rstn_o   : out   std_logic;
     fd_txck_o   : out   std_logic;
     fd_txd_o    : out   std_logic;
     fd_txena_o  : out   std_logic;

     -- Interrupts
     irq_p_o     : out   std_logic;

     -- Aux
     activate_loop_p_o : out std_logic;

    -- WISHBONE bus interface with the GN4124/VME core for the configuration of the core
     wb_adr_i    : in    std_logic_vector(g_span-1 downto 0);   -- WISHBONE classic address
     wb_dat_i    : in    std_logic_vector(g_width-1 downto 0);  -- WISHBONE classic data in
     wb_stb_i    : in    std_logic;                             -- WISHBONE classic strobe
     wb_we_i     : in    std_logic;                             -- WISHBONE classic write enable
     wb_cyc_i    : in    std_logic;                             -- WISHBONE classic cycle
     wb_ack_o    : out   std_logic;                             -- WISHBONE classic acknowledge
     wb_dat_o    : out   std_logic_vector(g_width-1 downto 0)); -- WISHBONE classic data out

end fmc_masterFIP_core;


--=================================================================================================
--                                    architecture declaration
--=================================================================================================
architecture rtl of fmc_masterFIP_core is

  -- control unit
  signal activate_loop_p, continuous_loop                 : std_logic;
  signal irq_p                                              : std_logic;
  signal tx_start_p, tx_request_byte_p                      : std_logic;
  signal byte_request_accepted_p, cons_bytes_excess, rx_rst : std_logic;
  signal var                                                : t_var;
  signal prod_data_lgth, prod_byte_index, cons_byte_index   : std_logic_vector (7 downto 0);
  signal s_ident_rp_dat                                     : ident_rp_dat;
  signal s_consu_rp_dat                                     : consu_rp_dat; 
  signal num_of_bourr, ident_id_dat_var_o, consu_id_dat_var : std_logic_vector (31 downto 0);
  signal ident_id_dat_sub, consu_id_dat_sub,ident_id_dat_var: std_logic_vector (31 downto 0);
   -- wf_fd_receiver outputs
  signal rx_fss_received_p, rx_fss_crc_fes_ok_p             : std_logic;
  signal rx_crc_wrong_p, rx_byte_ready_p                    : std_logic;
  signal rx_byte                                            : std_logic_vector (7 downto 0);
  -- wf_production outputs
  signal  byte_to_tx                                        : std_logic_vector (7 downto 0);
  -- wf_fd_transmitter outputs
  signal tx_last_byte_p, tx_completed_p, byte_request_accept_p   : std_logic;
  signal tx_byte_request_p, tx_byte_request_accept_p       : std_logic;
  signal tx_last_data_byte_p : std_logic;


  -- Chipscope
  --component chipscope_ila
  --  port (
  --    CONTROL : inout std_logic_vector(35 downto 0);
  --    CLK     : in    std_logic;
  --    TRIG0   : in    std_logic_vector(31 downto 0);
  --    TRIG1   : in    std_logic_vector(31 downto 0);
  --    TRIG2   : in    std_logic_vector(31 downto 0);
  --    TRIG3   : in    std_logic_vector(31 downto 0));
  --end component;
  --component chipscope_icon
  --  port (
  --    CONTROL0 : inout std_logic_vector (35 downto 0));
  --end component;

  signal CONTROL : std_logic_vector(35 downto 0);
  signal CLK     : std_logic;
  signal TRIG0   : std_logic_vector(31 downto 0);
  signal TRIG1   : std_logic_vector(31 downto 0);
  signal TRIG2   : std_logic_vector(31 downto 0);
  signal TRIG3   : std_logic_vector(31 downto 0);

  
--=================================================================================================
--                                       architecture begin
--=================================================================================================
begin

---------------------------------------------------------------------------------------------------
--                                     REGISTERS CONTROLLER                                      --
---------------------------------------------------------------------------------------------------
  reg_control_block: reg_ctrl
  generic map
    (g_span                => 32,
     g_width               => 32)
  port map
    (clk_i                 => clk_i,
     rst_i                 => rst_i,
     wb_adr_i              => wb_adr_i,
     wb_dat_i              => wb_dat_i,
     wb_stb_i              => wb_stb_i,
     wb_we_i               => wb_we_i,
     wb_cyc_i              => wb_cyc_i,
     wb_ack_o              => wb_ack_o,
     wb_dat_o              => wb_dat_o,
     activate_loop_p_o     => activate_loop_p,
     continuous_loop_o     => continuous_loop,
     ident_rp_dat_i        => s_ident_rp_dat,  
     consu_rp_dat_o        => s_consu_rp_dat,
     num_of_bourr_o        => num_of_bourr,
     ident_id_dat_var_o    => ident_id_dat_var,
     ident_id_dat_sub_o    => ident_id_dat_sub,
     consu_id_dat_sub_o    => consu_id_dat_sub,
     consu_id_dat_var_o    => consu_id_dat_var);

activate_loop_p_o <= activate_loop_p;

---------------------------------------------------------------------------------------------------
--                                          DATA ENGINE                                          --
---------------------------------------------------------------------------------------------------
  data_engine_block: data_engine
  port map
    (clk_i                      => clk_i,
     rst_i                      => rst_i,
     activate_loop_p_i          => activate_loop_p,
     continuous_loop_i          => continuous_loop,
     num_of_bourr_i             => num_of_bourr,

     tx_completed_p_i           => tx_completed_p,
     tx_byte_request_p_i        => tx_request_byte_p,

     rx_byte_ready_p_i          => rx_byte_ready_p,
     rx_fss_crc_fes_ok_p_i      => rx_fss_crc_fes_ok_p,
     rx_crc_wrong_p_i           => rx_crc_wrong_p,
     rx_fss_received_p_i        => rx_fss_received_p,

     tx_start_p_o               => tx_start_p,
     tx_byte_request_accept_p_o => tx_byte_request_accept_p,
     tx_last_data_byte_p_o      => tx_last_byte_p,

     rx_rst_o                   => rx_rst,
     cons_bytes_excess_o        => cons_bytes_excess,

     prod_byte_index_o          => prod_byte_index,

     cons_byte_index_o          => cons_byte_index,
     var_o                      => var,
     irq_p_o                    => irq_p);

  irq_p_o <= irq_p;


---------------------------------------------------------------------------------------------------
--                                         wf_consumption                                        --
---------------------------------------------------------------------------------------------------

  Consumption_Bytes_Processor : wf_cons_bytes_processor
  port map(
    uclk_i                 => clk_i,
    nfip_rst_i             => rst_i,
    var_i                  => var,
    byte_i                 => rx_byte,
    byte_ready_p_i         => rx_byte_ready_p,
    byte_index_i           => cons_byte_index,
   --------------------------------------------------------
    ident_rp_dat_o         => s_ident_rp_dat);
   --------------------------------------------------------


  FIELDRIVE_Receiver: wf_fd_receiver
  port map(
    uclk_i                => clk_i,
    nfip_rst_i            => rst_i,
    rate_i                => "01",
    fd_rxd_a_i            => fd_rxd_i,
    rx_rst_i              => rx_rst,
  -------------------------------------------------------------
    rx_byte_o             => rx_byte,
    rx_byte_ready_p_o     => rx_byte_ready_p,
    rx_fss_crc_fes_ok_p_o => rx_fss_crc_fes_ok_p,
    rx_fss_received_p_o   => rx_fss_received_p,
    rx_crc_wrong_p_o      => rx_crc_wrong_p);
  -------------------------------------------------------------


---------------------------------------------------------------------------------------------------
--                                         wf_production                                         --
---------------------------------------------------------------------------------------------------

  production_bytes_retriever : wf_prod_bytes_retriever
  port map(
    uclk_i                  => clk_i,
    nfip_rst_i              => rst_i,

    ident_id_dat_var_i      => ident_id_dat_var(7 downto 0),
    ident_id_dat_sub_i      => ident_id_dat_sub(7 downto 0),
    consu_id_dat_sub_i      => consu_id_dat_sub(7 downto 0),
    consu_id_dat_var_i      => consu_id_dat_var(7 downto 0),
    consu_rp_dat_i          => s_consu_rp_dat,
    consu_rp_dat_lgth_i     => "00000101", --consu_rp_dat_lgth(7 downto 0),
    var_i                   => var,
    byte_index_i            => prod_byte_index,
    byte_being_sent_p_i     => tx_byte_request_accept_p,
   -----------------------------------------------
    byte_o                  => byte_to_tx);
   -----------------------------------------------


  Produced_Data_Length_Calculator: wf_prod_data_lgth_calc
  port map(
    uclk_i             => clk_i,
    nfip_rst_i         => rst_i,
    p3_lgth_i          => "000",
    var_i              => var,
    -------------------------------------------------------
    prod_data_lgth_o   => prod_data_lgth);
    -------------------------------------------------------



  FIELDRIVE_Transmitter: wf_fd_transmitter
  port map(
    uclk_i                     => clk_i,
    nfip_rst_i                 => rst_i,
    rate_i                     => "01",
    tx_byte_i                  => byte_to_tx,
    tx_byte_request_accept_p_i => tx_byte_request_accept_p,
    tx_last_data_byte_p_i      => tx_last_byte_p,
    tx_start_p_i               => tx_start_p,
  -------------------------------------------------------------
    tx_byte_request_p_o        => tx_request_byte_p,
    tx_completed_p_o           => tx_completed_p,
    tx_data_o                  => fd_txd_o,
    tx_enable_o                => fd_txena_o,
    tx_clk_o                   => fd_txck_o);
  -------------------------------------------------------------



---------------------------------------------------------------------------------------------------
--                                           CHIPSCOPE                                           --
---------------------------------------------------------------------------------------------------   
  
--   chipscope_ila_1 : chipscope_ila
--   port map (
--     CONTROL => CONTROL,
--     CLK     => clk_i,
--     TRIG0   => TRIG0,
--     TRIG1   => TRIG1,
--     TRIG2   => TRIG2,
--     TRIG3   => TRIG3);

--  chipscope_icon_1 : chipscope_icon
--   port map (
--     CONTROL0 => CONTROL);
 
--  TRIG0(7 downto 0)   <= rx_byte;
--  TRIG0(8)            <= rx_byte_ready_p;
--  TRIG0(16 downto 9)  <= byte_to_tx;
--  TRIG0(17)           <= tx_start_p;
--  TRIG0(18)           <= rx_fss_crc_fes_ok_p;
--  TRIG0(19)           <= activate_loop_p;
--  TRIG0(20)           <= tx_completed_p;
--  TRIG0(21)           <= rx_fss_crc_fes_ok_p;
--  TRIG0(22)           <= rx_fss_received_p;
--  TRIG0(23)           <= rx_crc_wrong_p;

--  TRIG1(7 downto 0)   <= prod_byte_index;
--  TRIG1(15 downto 8)  <= cons_byte_index;
--  TRIG1(16)           <= irq_p;

--  TRIG0(16)           <= start_from_fpga;  
--  TRIG0(24 downto 17) <= retrig_nb_offset(7 downto 0);
--  TRIG0(31 downto 25) <= clk_i_cycles_offset(6 downto 0);  
--
--  TRIG1(30 downto 0)  <= acam_tstamp1(30 downto 0);
--  TRIG1(31)           <= acam_tstamp1_ok_p;
--
--  TRIG2(31 downto 0)  <= utc(31 downto 0); 
--  
--  TRIG3(0)            <= tdc_in_fpga_1_i;
--  TRIG3(1)            <= roll_over_incr_recent;
--  TRIG3(2)            <= wrabbit_tai_p_i;
--  TRIG3(31 downto 3)  <= current_retrig_nb(28 downto 0);
  
end rtl;
----------------------------------------------------------------------------------------------------
--  architecture ends
----------------------------------------------------------------------------------------------------
