--_________________________________________________________________________________________________
--                                                                                                |
--                                           |TDC core|                                           |
--                                                                                                |
--                                         CERN,BE/CO-HT                                          |
--________________________________________________________________________________________________|

---------------------------------------------------------------------------------------------------
--                                                                                                |
--                                    fmc_masterFIP_mezzanine                                     |
--                                                                                                |
---------------------------------------------------------------------------------------------------
-- File         fmc_masterFIP_mezzanine.vhd                                                       |
--                                                                                                |
-- Description  The unit combines                                                                 |
--                o the masterFIP core                                                            |
--                o the I2C core for the communication with the TDC board EEPROM                  |
--                o the OneWire core for the communication with the TDC board UniqueID&Thermeter  |
--                o the Embedded Interrupt Controller core that concentrates several interrupt    |
--                  sources into one WISHBONE interrupt request line.                             |
--                                                                                                |
--              For the interconnection between the GN4124/VME core and the different cores (TDC, |
--              I2C, 1W, EIC, timestamps memory) the unit instantiates an SDB crossbar.           |
--                                                                                                |
--              Note that the TDC core uses word addressing, whereas the GN4124/VME cores use byte|
--              addressing                                                                        |
--                                   _______________________________                              |
--                                 |       FMC TDC mezzanine        |                             |
--                                 |                                |                             |
--                                 | |   ________________    ___    |                             |
--                                 | |->|                |  |   |   |                             |
--                 ACAM chip <-->  |    | masterFIP core |  |   |   |   <-->                      |
--                                 | |--|________________|  | S |   |                             |
--                                 | |   ________________   |   |   |                             |
--                                 | |  |                |  |   |   |                             |
--               EEPROM chip <-->  | |  |    I2C core    |  |   |   |   <-->                      |
--                                 | |  |________________|  |   |   |                             |
--                                 | |   ________________   | D |   |          GN4124/VME core    |
--                                 | |  |                |  |   |   |                             |
--                   1W chip <-->  | |  |     1W core    |  |   |   |   <-->                      |
--                                 | |  |________________|  |   |   |                             |
--                                 | |   ________________   |   |   |                             |
--                                 | |  |                |  | B |   |                             |
--                                 | |->|       EIC      |  |   |   |   <-->                      |
--                                 |    |________________|  |___|   |                             |
--                                 |                                |                             |
--                                 |________________________________|                             |
--                                     ^                       ^                                  |
--                                     | 40 MHz            rst |                                  |
--                                                                                                |
--                         Figure 1: FMC masterFIP mezzanine architecture and                     |
--                          connection with the clks_rsts_manager unit                            |
--                                                                                                |
--                                                                                                |
--                                                                                                |
-- Authors      Evangelia Gousiou (Evangelia.Gousiou@cern.ch)                                     |
-- Date         07/2014                                                                           |
-- Version      v2                                                                                |
-- Depends on                                                                                     |
--                                                                                                |
----------------                                                                                  |
-- Last changes                                                                                   |
--     07/2014  v1  EG  First version                                                             |
--                                                                                                |
---------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------
--                               GNU LESSER GENERAL PUBLIC LICENSE                                |
--                              ------------------------------------                              |
-- This source file is free software; you can redistribute it and/or modify it under the terms of |
-- the GNU Lesser General Public License as published by the Free Software Foundation; either     |
-- version 2.1 of the License, or (at your option) any later version.                             |
-- This source is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;       |
-- without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      |
-- See the GNU Lesser General Public License for more details.                                    |
-- You should have received a copy of the GNU Lesser General Public License along with this       |
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html                     |
---------------------------------------------------------------------------------------------------

--=================================================================================================
--                                       Libraries & Packages
--=================================================================================================
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use work.masterFIP_pkg.all;
use work.gencores_pkg.all;
use work.wishbone_pkg.all;


--=================================================================================================
--                           Entity declaration for fmc_masterFIP_mezzanine
--=================================================================================================
entity fmc_masterFIP_mezzanine is
  generic
    (g_with_wrabbit_core : boolean := FALSE;
     g_span              : integer := 32;
     g_width             : integer := 32;
     values_for_simul    : boolean := FALSE);
  port
    -- TDC core
    (-- Clock & reset 62M5
     clk_sys_i           : in    std_logic; -- 40 MHz clock
     rst_sys_n_i         : in    std_logic; -- reset for 40 MHz logic
     -- FielDrive
     fd_rxcdn_i          : in    std_logic;
     fd_rxd_i            : in    std_logic;
     fd_txer_i           : in    std_logic;
     fd_wdgn_i           : in    std_logic;
     fd_rstn_o           : out   std_logic;
     fd_txck_o           : out   std_logic;
     fd_txd_o            : out   std_logic;
     fd_txena_o          : out   std_logic;
     -- Aux
     activate_loop_p_o  : out   std_logic;
     -- WISHBONE interface with the GN4124/VME_core
     -- for the core configuration | data retrieval | core interrupts | 1Wire | I2C 
     wb_adr_i           : in    std_logic_vector(31 downto 0);
     wb_dat_i           : in    std_logic_vector(31 downto 0);
     wb_cyc_i           : in    std_logic;
     wb_sel_i           : in    std_logic_vector(3 downto 0);
     wb_stb_i           : in    std_logic;
     wb_we_i            : in    std_logic;
     wb_dat_o           : out   std_logic_vector(31 downto 0);
     wb_ack_o           : out   std_logic;
     wb_stall_o         : out   std_logic;
     wb_irq_o           : out   std_logic;
    -- I2C EEPROM interface
     i2c_scl_o          : out   std_logic;
     i2c_scl_oen_o      : out   std_logic;
     i2c_scl_i          : in    std_logic;
     i2c_sda_oen_o      : out   std_logic;
     i2c_sda_o          : out   std_logic;
     i2c_sda_i          : in    std_logic;
    -- Aux
     irq_p_o            : out   std_logic;
    -- 1-Wire interface
     onewire_b          : inout std_logic);
end fmc_masterFIP_mezzanine;


--=================================================================================================
--                                    architecture declaration
--=================================================================================================
architecture rtl of fmc_masterFIP_mezzanine is

---------------------------------------------------------------------------------------------------
--                                         SDB CONSTANTS                                         --
---------------------------------------------------------------------------------------------------
  -- Note: All address in sdb and crossbar are BYTE addresses!

  -- Master ports on the wishbone crossbar
  constant c_NUM_WB_MASTERS       : integer := 4;
  constant c_WB_SLAVE_ONEWIRE     : integer := 0;  -- mezzanine board UnidueID&Thermometer 1-wire
  constant c_WB_SLAVE_CORE_CONFIG : integer := 1;  -- core configuration and data registers
  constant c_WB_SLAVE_EIC         : integer := 2;  -- interrupts
  constant c_WB_SLAVE_I2C         : integer := 3;  -- mezzanine board EEPROM I2C

  -- Slave port on the wishbone crossbar
  constant c_NUM_WB_SLAVES            : integer := 1;

  -- Wishbone master(s)
  constant c_WB_MASTER                : integer := 0;

  -- sdb header address
  constant c_SDB_ADDRESS              : t_wishbone_address := x"00000000";

  -- WISHBONE crossbar layout
  constant c_INTERCONNECT_LAYOUT      : t_sdb_record_array(3 downto 0) :=
    (0 => f_sdb_embed_device(c_ONEWIRE_SDB_DEVICE,    x"00010000"),
     1 => f_sdb_embed_device(c_TDC_CONFIG_SDB_DEVICE, x"00011000"),
     2 => f_sdb_embed_device(c_TDC_EIC_DEVICE,        x"00012000"),
     3 => f_sdb_embed_device(c_I2C_SDB_DEVICE,        x"00013000"));


---------------------------------------------------------------------------------------------------
--                                            Signals                                            --
---------------------------------------------------------------------------------------------------
  -- resets
  signal general_rst_n, rst_ref_0_n: std_logic;
  -- Wishbone buse(s) from crossbar master port(s)
  signal cnx_master_out            : t_wishbone_master_out_array(c_NUM_WB_MASTERS-1 downto 0);
  signal cnx_master_in             : t_wishbone_master_in_array (c_NUM_WB_MASTERS-1 downto 0);
  -- Wishbone buse(s) to crossbar slave port(s)
  signal cnx_slave_out             : t_wishbone_slave_out_array(c_NUM_WB_SLAVES-1 downto 0);
  signal cnx_slave_in              : t_wishbone_slave_in_array (c_NUM_WB_SLAVES-1 downto 0);
  -- Wishbone bus from additional registers
  signal xreg_slave_out            : t_wishbone_slave_out;
  signal xreg_slave_in             : t_wishbone_slave_in;
  -- WISHBONE addresses
  signal wb_adr           : std_logic_vector(31 downto 0);
  signal tdc_mem_wb_adr            : std_logic_vector(31 downto 0);
  -- 1-wire
  signal mezz_owr_en, mezz_owr_i   : std_logic_vector(0 downto 0);
  -- I2C
  signal sys_scl_in, sys_scl_out   : std_logic;
  signal sys_scl_oe_n, sys_sda_in  : std_logic;
  signal sys_sda_out, sys_sda_oe_n : std_logic;
  -- IRQ
  signal irq_p  : std_logic;
  -- WRabbit
  signal reg_to_wr, reg_from_wr    : std_logic_vector(31 downto 0);
  signal wrabbit_utc_p             : std_logic;
  signal wrabbit_synched           : std_logic;


--=================================================================================================
--                                       architecture begin
--=================================================================================================
begin


---------------------------------------------------------------------------------------------------
--                                     CSR WISHBONE CROSSBAR                                     --
---------------------------------------------------------------------------------------------------
-- CSR wishbone address decoder
--   0x10000 -> mezzanine board 1-Wire
--   0x11000 -> masterFIP core configuration and data
--   0x12000 -> EIC
--   0x13000 -> mezzanine board EEPROM I2C

 --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  cmp_sdb_crossbar : xwb_sdb_crossbar
  generic map
    (g_num_masters  => c_NUM_WB_SLAVES,
     g_num_slaves   => c_NUM_WB_MASTERS,
     g_registered   => true,
     g_wraparound   => true,
     g_layout       => c_INTERCONNECT_LAYOUT,
     g_sdb_addr     => c_SDB_ADDRESS)
  port map
    (clk_sys_i      => clk_sys_i,
     rst_n_i        => rst_sys_n_i,
     slave_i        => cnx_slave_in,
     slave_o        => cnx_slave_out,
     master_i       => cnx_master_in,
     master_o       => cnx_master_out);

  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  -- Unused wishbone signals
  wb_dat_o   <= cnx_slave_out(c_WB_MASTER).dat;
  wb_ack_o   <= cnx_slave_out(c_WB_MASTER).ack;
  wb_stall_o <= cnx_slave_out(c_WB_MASTER).stall;
  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  -- Connect crossbar slave port to entity port
  cnx_slave_in(c_WB_MASTER).adr <= wb_adr_i;
  cnx_slave_in(c_WB_MASTER).dat <= wb_dat_i;
  cnx_slave_in(c_WB_MASTER).sel <= wb_sel_i;
  cnx_slave_in(c_WB_MASTER).stb <= wb_stb_i;
  cnx_slave_in(c_WB_MASTER).we  <= wb_we_i;
  cnx_slave_in(c_WB_MASTER).cyc <= wb_cyc_i;
  
---------------------------------------------------------------------------------------------------
--                                         masterFIP CORE                                        --
---------------------------------------------------------------------------------------------------
  cmp_tdc_core: fmc_masterFIP_core
  generic map
    (g_span           => g_span,
     g_width          => g_width,
     values_for_simul => FALSE)
  port map
    (-- clks, rst
     clk_i            => clk_sys_i,
     rst_i            => not (rst_sys_n_i),
     -- FielDrive
     fd_rxcdn_i       => fd_rxcdn_i,
     fd_rxd_i         => fd_rxd_i,
     fd_txer_i        => fd_txer_i,
     fd_wdgn_i        => fd_wdgn_i,
     fd_rstn_o        => fd_rstn_o,
     fd_txck_o        => fd_txck_o,
     fd_txd_o         => fd_txd_o,
     fd_txena_o       => fd_txena_o,
     -- Interrupts
     irq_p_o          => irq_p,
     -- Aux
     activate_loop_p_o => activate_loop_p_o,
     -- WISHBONE CSR for core configuration
     wb_adr_i         => wb_adr,
     wb_dat_i         => cnx_master_out(c_WB_SLAVE_CORE_CONFIG).dat,
     wb_stb_i         => cnx_master_out(c_WB_SLAVE_CORE_CONFIG).stb,
     wb_we_i          => cnx_master_out(c_WB_SLAVE_CORE_CONFIG).we,
     wb_cyc_i         => cnx_master_out(c_WB_SLAVE_CORE_CONFIG).cyc,
     wb_dat_o         => cnx_master_in(c_WB_SLAVE_CORE_CONFIG).dat,
     wb_ack_o         => cnx_master_in(c_WB_SLAVE_CORE_CONFIG).ack);

  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  -- Convert byte address into word address
  wb_adr <= "00" & cnx_master_out(c_WB_SLAVE_CORE_CONFIG).adr(31 downto 2);

  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  -- Unused wishbone signals
  cnx_master_in(c_WB_SLAVE_CORE_CONFIG).err   <= '0';
  cnx_master_in(c_WB_SLAVE_CORE_CONFIG).rty   <= '0';
  cnx_master_in(c_WB_SLAVE_CORE_CONFIG).stall <= '0';
  cnx_master_in(c_WB_SLAVE_CORE_CONFIG).int   <= '0';


---------------------------------------------------------------------------------------------------
--                        TDC Mezzanine Board UniqueID&Thermometer OneWire                       --
---------------------------------------------------------------------------------------------------
  cmp_fmc_onewire : xwb_onewire_master
  generic map
    (g_interface_mode      => PIPELINED,
     g_address_granularity => BYTE,
     g_num_ports           => 1,
     g_ow_btp_normal       => "5.0",
     g_ow_btp_overdrive    => "1.0")
  port map
    (clk_sys_i             => clk_sys_i,
     rst_n_i               => rst_sys_n_i,
     slave_i               => cnx_master_out(c_WB_SLAVE_ONEWIRE),
     slave_o               => cnx_master_in(c_WB_SLAVE_ONEWIRE),
     desc_o                => open,
     owr_pwren_o           => open,
     owr_en_o              => mezz_owr_en,
     owr_i                 => mezz_owr_i);
  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  onewire_b                <= '0' when mezz_owr_en(0) = '1' else 'Z';
  mezz_owr_i(0)            <= onewire_b;


---------------------------------------------------------------------------------------------------
--                             WBGEN2 EMBEDDED INTERRUPTS CONTROLLER                             --
---------------------------------------------------------------------------------------------------
-- IRQ sources
-- 0 -> macrocycle finished

  cmp_tdc_eic : tdc_eic
  port map
    (clk_sys_i          => clk_sys_i,
     rst_n_i            => rst_sys_n_i,
     wb_adr_i           => cnx_master_out(c_WB_SLAVE_EIC).adr(3 downto 2),
     wb_dat_i           => cnx_master_out(c_WB_SLAVE_EIC).dat,
     wb_dat_o           => cnx_master_in(c_WB_SLAVE_EIC).dat,
     wb_cyc_i           => cnx_master_out(c_WB_SLAVE_EIC).cyc,
     wb_sel_i           => cnx_master_out(c_WB_SLAVE_EIC).sel,
     wb_stb_i           => cnx_master_out(c_WB_SLAVE_EIC).stb,
     wb_we_i            => cnx_master_out(c_WB_SLAVE_EIC).we,
     wb_ack_o           => cnx_master_in(c_WB_SLAVE_EIC).ack,
     wb_stall_o         => cnx_master_in(c_WB_SLAVE_EIC).stall,
     wb_int_o           => wb_irq_o,
     irq_tdc_tstamps_i  => irq_p,
     irq_tdc_time_i     => '0',
     irq_tdc_acam_err_i => '0');

irq_p_o <= irq_p;

  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  -- Unused wishbone signals
  cnx_master_in(c_WB_SLAVE_EIC).err <= '0';
  cnx_master_in(c_WB_SLAVE_EIC).rty <= '0';
  cnx_master_in(c_WB_SLAVE_EIC).int <= '0';


---------------------------------------------------------------------------------------------------
--                                TDC Mezzanine Board EEPROM I2C                                 --
---------------------------------------------------------------------------------------------------
   cmp_I2C_master : xwb_i2c_master
   generic map
     (g_interface_mode      => PIPELINED,
      g_address_granularity => BYTE)
   port map
     (clk_sys_i             => clk_sys_i,
      rst_n_i               => rst_sys_n_i,
      slave_i               => cnx_master_out(c_WB_SLAVE_I2C),
      slave_o               => cnx_master_in(c_WB_SLAVE_I2C),
      desc_o                => open,
      scl_pad_i             => i2c_scl_i,
      scl_pad_o             => sys_scl_out,
      scl_padoen_o          => sys_scl_oe_n,
      sda_pad_i             => i2c_sda_i,
      sda_pad_o             => sys_sda_out,
      sda_padoen_o          => sys_sda_oe_n);

  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  i2c_sda_oen_o            <= sys_sda_oe_n;
  i2c_sda_o                <= sys_sda_out;
  i2c_scl_oen_o            <= sys_scl_oe_n;
  i2c_scl_o                <= sys_scl_out;


end rtl;
----------------------------------------------------------------------------------------------------
--  architecture ends
----------------------------------------------------------------------------------------------------