--_________________________________________________________________________________________________
--                                                                                                |
--                                           |TDC core|                                           |
--                                                                                                |
--                                         CERN,BE/CO-HT                                          |
--________________________________________________________________________________________________|

---------------------------------------------------------------------------------------------------
--                                                                                                |
--                                           reg_ctrl                                             |
--                                                                                                |
---------------------------------------------------------------------------------------------------
-- File         reg_ctrl.vhd                                                                      |
--                                                                                                |
-- Description  Interfaces with the GN4124/VME core for the configuration of the ACAM chip and of |
--              the TDC core. Data transfers take place between the GN4124/VME interface and      |
--              locally the TDC core. The unit implements a WISHBONE slave.                       |
--                                                                                                |
--              Through WISHBONE writes, the unit receives:                                       |
--                o the ACAM configuration registers which are then made available to the         |
--                  data_engine and acam_databus_interface units to be transferred to the ACAMchip|
--                o the local configuration registers (eg irq_thresholds, channels_enable, WRabbit|
--                  core enable) that are then made available to the different units of the design|
--                o the control register that defines the action to be taken in the core; the     |
--                  register is decoded and the corresponding signals are made available to the   |
--                  different units in the design.                                                |
--                                                                                                |
--              Through WISHBONE reads, the unit transmits:                                       |
--                o the ACAM configuration registers readback from the ACAM chip                  |
--                o status registers coming from different units of the TDC core                  |
--                                                                                                |
--              All the registers are of size 32 bits, as the WISHBONE data bus                   |
--                                                                                                |
--                                                                                                |
-- Authors      Gonzalo Penacoba  (Gonzalo.Penacoba@cern.ch)                                      |
--              Evangelia Gousiou (Evangelia.Gousiou@cern.ch)                                     |
-- Date         04/2014                                                                           |
-- Version      v2                                                                                |
-- Depends on                                                                                     |
--                                                                                                |
----------------                                                                                  |
-- Last changes                                                                                   |
--     10/2011  v0.1  GP  First version                                                           |
--     04/2012  v0.11 EG  Revamping; Comments added, signals renamed                              |
--     08/2012  v1    EG  added register reg_adr_pipe0 for slack timing reasons                   |
--     04/2014  v2    EG  added WRabbit register; added channel deactivation register             |
--                                                                                                |
---------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------
--                               GNU LESSER GENERAL PUBLIC LICENSE                                |
--                              ------------------------------------                              |
-- This source file is free software; you can redistribute it and/or modify it under the terms of |
-- the GNU Lesser General Public License as published by the Free Software Foundation; either     |
-- version 2.1 of the License, or (at your option) any later version.                             |
-- This source is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;       |
-- without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      |
-- See the GNU Lesser General Public License for more details.                                    |
-- You should have received a copy of the GNU Lesser General Public License along with this       |
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html                     |
---------------------------------------------------------------------------------------------------



--=================================================================================================
--                                       Libraries & Packages
--=================================================================================================

-- Standard library
library IEEE;
use IEEE.STD_LOGIC_1164.all; -- std_logic definitions
use IEEE.NUMERIC_STD.all;    -- conversion functions-- Specific library
-- Specific library
library work;
use work.masterFIP_pkg.all;   -- definitions of types, constants, entities


--=================================================================================================
--                            Entity declaration for reg_ctrl
--=================================================================================================

entity reg_ctrl is
  generic
    (g_span                : integer := 32;
     g_width               : integer := 32);
  port
  -- INPUTS
     -- Signals from the clks_rsts_manager unit
    (clk_i                 : in std_logic;                             -- 40 MHz
     rst_i                 : in std_logic;                             -- global reset, synced to clk_i

     -- Signals from the GN4124/VME_core unit: WISHBONE for regs transfer
     wb_adr_i              : in std_logic_vector(g_span-1 downto 0);   -- WISHBONE address
     wb_cyc_i              : in std_logic;                             -- WISHBONE cycle
     wb_dat_i              : in std_logic_vector(g_width-1 downto 0);  -- WISHBONE data in
     wb_stb_i              : in std_logic;                             -- WISHBONE strobe
     wb_we_i               : in std_logic;                             -- WISHBONE write enable

     ident_rp_dat_i        : in ident_rp_dat;                          -- array keeping values read back from 

  -- OUTPUTS
     -- Signals to the GN4124/VME_core unit: WISHBONE for regs transfer
     wb_ack_o              : out std_logic;                            -- WISHBONE acknowledge
     wb_dat_o              : out std_logic_vector(g_width-1 downto 0); -- WISHBONE data out

     -- Signals to the data_engine unit: TDC core functionality
     activate_loop_p_o     : out std_logic;                            -- activates loop
     continuous_loop_o     : out std_logic;                            -- 

     -- Signals from the 
     consu_rp_dat_o        : out consu_rp_dat;                          -- array keeping values read back from 
     num_of_bourr_o        : out std_logic_vector(g_width-1 downto 0);
     ident_id_dat_var_o    : out std_logic_vector(g_width-1 downto 0);
     ident_id_dat_sub_o    : out std_logic_vector(g_width-1 downto 0);
     consu_id_dat_sub_o    : out std_logic_vector(g_width-1 downto 0);
     consu_id_dat_var_o    : out std_logic_vector(g_width-1 downto 0));

end reg_ctrl;


--=================================================================================================
--                                    architecture declaration
--=================================================================================================
architecture rtl of reg_ctrl is

  signal reg_adr,reg_adr_pipe0                        : std_logic_vector(7 downto 0);
  signal ident_id_dat_sub, ident_id_dat_var, consu_id_dat_var, num_of_bourr    : std_logic_vector(g_width-1 downto 0);
  signal ctrl_reg, consu_id_dat_sub, irq_tstamp_threshold : std_logic_vector(g_width-1 downto 0);
  signal irq_time_threshold                           : std_logic_vector(g_width-1 downto 0);
  signal clear_ctrl_reg, send_dac_word_p              : std_logic;
  signal dac_word                                     : std_logic_vector(23 downto 0);
  signal pulse_extender_en                            : std_logic;
  signal pulse_extender_c                             : std_logic_vector(2 downto 0);
  signal dat_out, wrabbit_ctrl_reg, deactivate_chan   : std_logic_vector(g_span-1 downto 0);
  signal wb_ack_o_pipe0                    : std_logic;
  signal s_consu_rp_dat : consu_rp_dat;


--=================================================================================================
--                                       architecture begin
--=================================================================================================
begin

  reg_adr <= wb_adr_i(7 downto 0); -- we are interested in addresses 00 to FC


---------------------------------------------------------------------------------------------------
--                                WISHBONE ACK to GN4124/VME_core                                --
---------------------------------------------------------------------------------------------------
--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
-- TDCconfig_ack_generator: generation of the WISHBONE acknowledge signal for the
-- interactions with the GN4124/VME_core.

  TDCconfig_ack_generator: process (clk_i)
  begin
    if rising_edge (clk_i) then
      if rst_i = '1' then
        wb_ack_o <= '0';
        wb_ack_o_pipe0 <= '0';
      elsif(wb_cyc_i = '0') then
        wb_ack_o <= '0';
        wb_ack_o_pipe0 <= '0';
      else
        wb_ack_o <= wb_ack_o_pipe0;
        wb_ack_o_pipe0 <= wb_stb_i and wb_cyc_i;
      end if;
    end if;
  end process;


---------------------------------------------------------------------------------------------------
--                           Reception of core Configuration Registers                           --
---------------------------------------------------------------------------------------------------
--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
-- TDCcore_config_reg_reception: reception from the GN4124/VME interface of the configuration
-- registers to be loaded locally.
-- The following information is received:
--   o ident_id_dat_var : 
--   o ident_id_dat_sub :
--   o consu_id_dat_sub :
--   o consu_id_dat_var :

  TDCcore_config_reg_reception: process (clk_i)
  begin
    if rising_edge (clk_i) then
      if rst_i ='1' then
        ident_id_dat_var   <= (others =>'0');
        ident_id_dat_sub   <= (others =>'0');
        consu_id_dat_var   <= (others =>'0');
        consu_id_dat_sub   <= (others =>'0');
        s_consu_rp_dat(0)  <= (others =>'0');
        s_consu_rp_dat(1)  <= (others =>'0');
        num_of_bourr       <= (others =>'0');

      elsif wb_cyc_i = '1' and wb_stb_i = '1' and wb_we_i = '1' then -- WISHBONE writes

        if reg_adr = c_IDENT_ID_DAT_VAR_ADR then
          ident_id_dat_var <= wb_dat_i;
        end if;

        if reg_adr = c_IDENT_ID_DAT_SUB_ADR then
          ident_id_dat_sub <= wb_dat_i;
        end if;

        if reg_adr = c_CONSU_ID_DAT_VAR_ADR then
          consu_id_dat_var <= wb_dat_i;
        end if;

        if reg_adr = c_CONSU_ID_DAT_SUB_ADR then
          consu_id_dat_sub <= wb_dat_i;
        end if;

        if reg_adr = c_CONSU_RP_DAT_BYTE0_ADR then
          s_consu_rp_dat(0) <= wb_dat_i;
        end if;

        if reg_adr = c_CONSU_RP_DAT_BYTE1_ADR then
          s_consu_rp_dat(1) <= wb_dat_i;
        end if;

        if reg_adr = c_NUM_OF_BOURR_ADR then
          num_of_bourr <= wb_dat_i;
        end if;

      end if;
    end if;
  end process;
  --  --  --  --  --  --  --  --  --  --  --  --
  ident_id_dat_var_o       <= ident_id_dat_var;
  ident_id_dat_sub_o       <= ident_id_dat_sub;
  consu_id_dat_var_o       <= consu_id_dat_var;
  consu_id_dat_sub_o       <= consu_id_dat_sub;
  consu_rp_dat_o           <= s_consu_rp_dat;
  num_of_bourr_o           <= num_of_bourr;

  continuous_loop_o        <= num_of_bourr(31);


---------------------------------------------------------------------------------------------------
--                             Reception of TDC core Control Register                            --
---------------------------------------------------------------------------------------------------    
--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
-- TDCcore_ctrl_reg_reception: reception from the GN4124/VME interface of the control register that
-- defines the action to be taken by the TDC core.
-- Note that only one bit of the register should be written at a time. The process receives
-- the register, defines the action to be taken and after 1 clk cycle clears the register. 
    
  TDCcore_ctrl_reg_reception: process (clk_i)
  begin
    if rising_edge (clk_i) then
      if rst_i = '1' then
        ctrl_reg         <= (others =>'0');
        clear_ctrl_reg   <= '0';

      elsif clear_ctrl_reg = '1' then
        ctrl_reg         <= (others =>'0');
        clear_ctrl_reg   <= '0';

      elsif wb_cyc_i = '1' and wb_stb_i = '1' and wb_we_i = '1' then -- WISHBONE writes
        if reg_adr = c_CTRL_REG_ADR then
          ctrl_reg       <= wb_dat_i;
          clear_ctrl_reg <= '1';
        end if;

      end if;
    end if;
  end process;
  --  --  --  --  --  --  --  --  --  --  --  --   
  activate_loop_p_o       <= ctrl_reg(0);


---------------------------------------------------------------------------------------------------
--                      Delivery of WorldFIP consumed data and core Registers                    --
---------------------------------------------------------------------------------------------------   
-- TDCcore_ctrl_reg_reception: Delivery to the GN4124/VME interface of all the readable registers,
-- including those of the ACAM and the TDC core.
-- Note: pipelining of the address for timing/slack reasons 

  WISHBONEreads: process (clk_i)
  begin
    if rising_edge (clk_i) then
      --if wb_cyc_i = '1' and wb_stb_i = '1' and wb_we_i = '0' then -- WISHBONE reads
      --  wb_dat_o <= dat_out;
      reg_adr_pipe0       <= reg_adr;
      wb_dat_o <= dat_out;
      --end if;
    end if;
  end process;

--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  with reg_adr_pipe0 select dat_out <= 

    -- regs written by the GN4124/VME interface
    consu_id_dat_var  when c_CONSU_ID_DAT_VAR_ADR,
    consu_id_dat_sub  when c_CONSU_ID_DAT_SUB_ADR,
    ident_id_dat_sub  when c_IDENT_ID_DAT_SUB_ADR,
    ident_id_dat_var  when c_IDENT_ID_DAT_VAR_ADR,

    -- regs written by the WorldFIP interface for the Identification variable
    ident_rp_dat_i(0) when c_IDENT_RP_DAT_BYTE0_ADR,
    ident_rp_dat_i(1) when c_IDENT_RP_DAT_BYTE1_ADR,
    ident_rp_dat_i(2) when c_IDENT_RP_DAT_BYTE2_ADR,
    ident_rp_dat_i(3) when c_IDENT_RP_DAT_BYTE3_ADR,
    ident_rp_dat_i(4) when c_IDENT_RP_DAT_BYTE4_ADR,
    ident_rp_dat_i(5) when c_IDENT_RP_DAT_BYTE5_ADR,
    ident_rp_dat_i(6) when c_IDENT_RP_DAT_BYTE6_ADR,
    ident_rp_dat_i(7) when c_IDENT_RP_DAT_BYTE7_ADR,
    ident_rp_dat_i(8) when c_IDENT_RP_DAT_BYTE8_ADR,
    ident_rp_dat_i(9) when c_IDENT_RP_DAT_BYTE9_ADR,

    -- regs written by the worldFIP interface for the Consumed variable
    s_consu_rp_dat(0)   when c_CONSU_RP_DAT_BYTE0_ADR,
    s_consu_rp_dat(1)   when c_CONSU_RP_DAT_BYTE1_ADR,

    num_of_bourr        when c_NUM_OF_BOURR_ADR,

    -- others
    x"C0FFEEEE"            when others;


end architecture rtl;
--=================================================================================================
--                                        architecture end
--=================================================================================================
---------------------------------------------------------------------------------------------------
--                                      E N D   O F   F I L E
---------------------------------------------------------------------------------------------------