--_________________________________________________________________________________________________
--                                                                                                |
--                                         |The nanoFIP|                                          |
--                                                                                                |
--                                         CERN,BE/CO-HT                                          |
--________________________________________________________________________________________________|

---------------------------------------------------------------------------------------------------
--                                                                                                |
--                                     wf_cons_bytes_processor                                    |
--                                                                                                |
---------------------------------------------------------------------------------------------------
-- File         wf_cons_bytes_processor.vhd                                                       |
--                                                                                                |
-- Description  The unit is consuming the RP_DAT data bytes that are arriving from the            |
--              wf_fd_receiver, according to the following:                                       |
--                                                                                                |
--              o If the variable identifier of the preceded ID_DAT was:                          |
--                                                                                                |
--               var_1 or var_2                                                                   |
--                - If the operation is in memory mode    : the unit is registering the           |
--                  application-data bytes along with the PDU_TYPE, LGTH and MPS bytes in the     |
--                  Consumed or Broadcast Consumed memories                                       |
--                - If the operation is in stand-alone mode: the unit is transferring the 2 appli-|
--                  cation-data bytes to the "nanoFIP User Interface, NON_WISHBONE" data bus DAT_O|
--                                                                                                |
--              o var_rst                                                                         |
--                 the two application-data bytes are identified and sent to the wf_reset_unit.   |
--                                                                                                |
--              o var_5                                                                           |
--                 regardless of the operational mode (memory or slone) the unit is registering   |
--                 the application-data bytes along with the PDU_TYPE, LGTH and MPS bytes in the  |
--                 JC_consumed memory.                                                            |
--                                                                                                |
--              Note: The validity of the consumed bytes (stored in the memories or transfered    |
--              to DATO or transfered to the wf_reset_unit) is indicated by the "nanoFIP User     |
--              Interface, NON_WISHBONE" signals VAR1_RDY/ VAR2_RDY or the nanoFIP internal       |
--              signals rst_nFIP_and_FD_p/ assert_RSTON_p/ start_jc_i, which are treated in       |
--              the wf_cons_outcome unit and are assessed after the end of the reception of a     |
--              complete frame.                                                                   |
--                                                                                                |
--              Reminder:                                                                         |
--                                                                                                |
--              Consumed RP_DAT frame structure :                                                 |
--       ___________ ______  _______ ________ __________________ _______  ___________ _______     |
--      |____FSS____|_CTRL_||__PDU__|__LGTH__|__..ApplicData..__|__MPS__||____FCS____|__FES__|    |
--                                                                                                |
--                                           |--------&LGTH bytes-------|                         |
--                          |--------write to Consumed memories---------|                         |
--                                           |-----to DAT_O-----|                                 |
--                                           |---to Reset Unit--|                                 |
--                                                                                                |
-- Authors      Pablo Alvarez Sanchez (Pablo.Alvarez.Sanchez@cern.ch)                             |
--              Evangelia Gousiou     (Evangelia.Gousiou@cern.ch)                                 |
-- Date         15/12/2010                                                                        |
-- Version      v0.03                                                                             |
-- Depends on   wf_reset_unit                                                                     |
--              wf_fd_receiver                                                                    |
--              wf_engine_control                                                                 |
----------------                                                                                  |
-- Last changes                                                                                   |
--     11/09/2009  v0.01  EB  First version                                                       |
--        09/2010  v0.02  EG  Treatment of reset variable added; Bytes_Transfer_To_DATO unit      |
--                            creation for simplification; Signals renamed;                       |
--                            CTRL, PDU_TYPE, LGTH bytes registered;                              |
--                            Code cleaned-up & commented.                                        |
--     15/12/2010  v0.03  EG  Unit renamed from wf_cons_bytes_from_rx to wf_cons_bytes_processor  |
--                            byte_ready_p comes from the rx_deserializer (no need to pass from   |
--                            the engine) Code cleaned-up & commented (more!)                     |
---------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------
--                               GNU LESSER GENERAL PUBLIC LICENSE                                |
--                             -------------------------------------                              |
-- This source file is free software; you can redistribute it and/or modify it under the terms of |
-- the GNU Lesser General Public License as published by the Free Software Foundation; either     |
-- version 2.1 of the License, or (at your option) any later version.                             |
-- This source is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;       |
-- without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      |
-- See the GNU Lesser General Public License for more details.                                    |
-- You should have received a copy of the GNU Lesser General Public License along with this       |
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html                     |
---------------------------------------------------------------------------------------------------



--=================================================================================================
--                                       Libraries & Packages
--=================================================================================================

-- Standard library
library IEEE;
use IEEE.STD_LOGIC_1164.all; -- std_logic definitions
use IEEE.NUMERIC_STD.all;    -- conversion functions
-- Specific library
library work;
use work.masterFIP_pkg.all;     -- definitions of types, constants, entities


--=================================================================================================
--                            Entity declaration for wf_cons_bytes_processor
--=================================================================================================
entity wf_cons_bytes_processor is port(
  -- INPUTS
    -- nanoFIP User Interface, General signals
    uclk_i                : in std_logic;                       -- 40 MHz clock

    -- Signal from the wf_reset_unit
    nfip_rst_i            : in std_logic;                       -- nanoFIP internal reset

    -- Signals from the wf_fd_receiver unit
    byte_i                : in std_logic_vector (7 downto 0);   -- input byte
    byte_ready_p_i        : in std_logic;                       -- indication of a new input byte

    -- Signals from the wf_engine_control unit
    byte_index_i          : in std_logic_vector (7 downto 0);   -- index of a byte inside the frame;
                                                                -- starting from 0, it counts all the
                                                                -- bytes after the FSS&before the FES

    var_i                 : in t_var;                           -- variable type that is being treated


  -- OUTPUTS
    -- nanoFIP User Interface, WISHBONE Slave output

    ident_rp_dat_o        : out ident_rp_dat);                   -- array keeping values from identification var 

    -- Signals to the wf_cons_outcome unit
    -- cons_ctrl_byte_o      : out std_logic_vector (7 downto 0);  -- received RP_DAT CTRL byte
    -- cons_lgth_byte_o      : out std_logic_vector (7 downto 0);  -- received RP_DAT LGTH byte
    -- cons_pdu_byte_o       : out std_logic_vector (7 downto 0);  -- received RP_DAT PDY_TYPE byte
    -- cons_var_rst_byte_1_o : out std_logic_vector (7 downto 0);  -- received var_rst RP_DAT, 1st data byte
    -- cons_var_rst_byte_2_o : out std_logic_vector (7 downto 0)); -- received var_rst RP_DAT, 2nd data byte

end entity wf_cons_bytes_processor;


--=================================================================================================
--                                    architecture declaration
--=================================================================================================
architecture rtl of wf_cons_bytes_processor is

  -- addressing the memory
  signal s_base_adr       : unsigned (8 downto 0);
  signal s_adr            : std_logic_vector (8 downto 0);
  -- bus/ memories write enable
  signal s_slone_wr_en_p  : std_logic_vector (1 downto 0);
  signal s_mem_wr_en_p    : std_logic;
  signal s_jc_mem_wr_en_p : std_logic;
  -- data bytes
  signal s_slone_data_out : std_logic_vector (15 downto 0);
  signal s_mem_data_out   : std_logic_vector (7 downto 0);
  -- Length byte
  signal s_cons_lgth_byte : std_logic_vector (7 downto 0);


--=================================================================================================
--                                       architecture begin
--=================================================================================================
begin


---------------------------------------------------------------------------------------------------
--                         Memory mode Consumed & Consumed Broadcast RAM                         --
--               Storage (by the unit) & retrieval (by the user) of consumed bytes               --
---------------------------------------------------------------------------------------------------
-- Instantiation of a 512 x 8  Dual Port RAM, for both the consumed and consumed broadcast vars
-- Port A is connected to the WISHBONE interface for the readings from the user
-- Port B is used by the nanoFIP for the writings into the memory

  -- Consumption_RAM : wf_dualram_512x8_clka_rd_clkb_wr
  -- port map(
    -- clk_porta_i      => wb_clk_i,	        -- WISHBONE clock
    -- addr_porta_i     => wb_adr_i,           -- address of byte to be read
    -- clk_portb_i      => uclk_i,             -- 40 MHz clock
    -- addr_portb_i     => s_adr,              -- address of byte to be written
    -- data_portb_i     => byte_i,             -- byte to be written
    -- write_en_portb_i => s_mem_wr_en_p,      -- write enable
   -- --------------------------------------------
    -- data_porta_o     => s_mem_data_out);    -- output byte read
   -- --------------------------------------------


---------------------------------------------------------------------------------------------------
--                         Slone mode Storage of consumed bytes to DATO                          --
---------------------------------------------------------------------------------------------------
-- Synchronous process Data_Transfer_To_Dat_o: In stand-alone mode, according to the signal
-- s_slone_wr_en_p, the first or second byte of the "User Interface, NON WISHBONE" bus DAT_O
-- takes the byte byte_i.

  Data_Transfer_To_Regs: process (uclk_i)
  begin
    if rising_edge (uclk_i) then
      if nfip_rst_i = '1' then                     -- bus initialization
        ident_rp_dat_o(0)       <= (others => '0');
		  ident_rp_dat_o(1)       <= (others => '0');
        ident_rp_dat_o(2)       <= (others => '0');
        ident_rp_dat_o(3)       <= (others => '0');
		  ident_rp_dat_o(4)       <= (others => '0');
		  ident_rp_dat_o(5)       <= (others => '0');
		  ident_rp_dat_o(6)       <= (others => '0');
		  ident_rp_dat_o(7)       <= (others => '0');
		  ident_rp_dat_o(8)       <= (others => '0');
		  ident_rp_dat_o(9)       <= (others => '0');

      else

        if var_i = rp_dat_ident then

          if byte_ready_p_i = '1' then   

            if byte_index_i = c_PDU_BYTE_INDEX then
              ident_rp_dat_o(0) <= x"000000" & byte_i;

            elsif byte_index_i = c_LGTH_BYTE_INDEX then
              ident_rp_dat_o(1) <= x"000000" & byte_i;

            elsif byte_index_i = c_1st_DATA_BYTE_INDEX then
              ident_rp_dat_o(2) <= x"000000" & byte_i;

            elsif byte_index_i = c_2nd_DATA_BYTE_INDEX then
              ident_rp_dat_o(3) <= x"000000" & byte_i;

            elsif byte_index_i = c_3rd_DATA_BYTE_INDEX then
              ident_rp_dat_o(4) <= x"000000" & byte_i;

            elsif byte_index_i = c_CONSTR_BYTE_INDEX then
              ident_rp_dat_o(5) <= x"000000" & byte_i;

            elsif byte_index_i = c_MODEL_BYTE_INDEX then
              ident_rp_dat_o(6) <= x"000000" & byte_i;

            elsif byte_index_i = c_6th_DATA_BYTE_INDEX then
              ident_rp_dat_o(7) <= x"000000" & byte_i;

            elsif byte_index_i = c_7th_DATA_BYTE_INDEX then
              ident_rp_dat_o(8) <= x"000000" & byte_i;

            elsif byte_index_i = c_8th_DATA_BYTE_INDEX then
              ident_rp_dat_o(9) <= x"000000" & byte_i;
            end if;
				
          end if;
        end if;
      end if;
    end if;
  end process;


---------------------------------------------------------------------------------------------------
--                                    CTRL, PDU_TYPE, LGTH bytes                                 --
---------------------------------------------------------------------------------------------------
-- Synchronous process Register_CTRL_PDU_LGTH_bytes: Storage of the CTRL, PDU_TYPE
-- and LGTH bytes of an incoming RP_DAT frame. The bytes are sent to the wf_cons_outcome
-- unit that validates them and accordingly activates the:
-- VAR1_RDY, for a var_1,
-- VAR2_RDY, for a var_2,
-- assert_rston_p & rst_nfip_and_fd_p, for a var_rst,
-- jc_start_p, for a var_4.

  -- Register_CTRL_PDU_LGTH_bytes: process (uclk_i)
  -- begin

    -- if rising_edge (uclk_i) then
      -- if nfip_rst_i = '1' then
        -- cons_ctrl_byte_o     <= (others => '0');
        -- cons_pdu_byte_o      <= (others => '0');
        -- s_cons_lgth_byte     <= (others => '0');

      -- else

        -- if (var_i = var_1) or (var_i = var_2) or (var_i = var_rst) or (var_i = var_4)then  -- only for consumed vars

          -- if (byte_index_i = c_CTRL_BYTE_INDEX) and (byte_ready_p_i='1') then
            -- cons_ctrl_byte_o <= byte_i;

          -- elsif (byte_index_i = c_PDU_BYTE_INDEX) and (byte_ready_p_i ='1') then
            -- cons_pdu_byte_o  <= byte_i;

          -- elsif (byte_index_i = c_LGTH_BYTE_INDEX) and (byte_ready_p_i ='1') then
            -- s_cons_lgth_byte <= byte_i;
          -- end if;

        -- else
          -- cons_ctrl_byte_o   <= (others => '0');
          -- cons_pdu_byte_o    <= (others => '0');
          -- s_cons_lgth_byte   <= (others => '0');
        -- end if;
      -- end if;
    -- end if;
  -- end process;

  -- --  --  --  --  --  --  --  --  --  --  --  --
  -- cons_lgth_byte_o         <= s_cons_lgth_byte;


end architecture rtl;
--=================================================================================================
--                                        architecture end
--=================================================================================================
---------------------------------------------------------------------------------------------------
--                                      E N D   O F   F I L E
---------------------------------------------------------------------------------------------------